from django.core.mail import mail_admins
from django.template import Context
from django.template.loader import get_template


class Reporter(object):
    def __init__(self, subject, template, base_context=None, committer=None):
        self.subject = subject
        self.template = template
        self.stored_text = []
        self.base_context = base_context if base_context is not None else {}
        self.comitter = committer if committer is not None else admin_mail_comitter

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.commit()

    def append(self, text):
        """ Adding a line
        """
        self.stored_text.append(text)

    def extend(self, text_list):
        """ Adding multiple lines
        """
        self.stored_text.extend(text_list)

    def render(self):
        """ Render the text for getting report text
        """
        ctx = self.base_context.copy()
        ctx['stored_text'] = self.stored_text
        return get_template(self.template).render(Context(ctx))

    def commit(self):
        """ A interface to send the report
        """
        self.comitter(self.subject, self.render())


def console_comitter(subject, body):
    print(subject)
    print(body)


def admin_mail_comitter(subject, body):
    mail_admins(subject, body, fail_silently=True)



from functools import wraps
import traceback

from reportmail.reporter import Reporter


def apply_reporter(subject, template='reportmail/command_report.txt'):
    """ Adding a reporting feature for django command

    You can use thin as decorator for Command.handle.
    and decorated handle() will get admin mail reporter objects as first argument::

        @reporter("Title of report", 'path/to/template.txt')
        def handle(reporter, *args, **options):
            ...

    * :arg subject: Title of report
    * :arg template: Template to use rendering
    """
    def wrapper(handle_func):
        @wraps(handle_func)
        def wrapped(self, *args, **options):
            with Reporter(subject, template,
                          base_context={'args': args, 'options': options,
                                        'command': self.__module__}) as reporter:
                try:
                    ret = handle_func(self, reporter, *args, **options)
                except Exception as e:
                    reporter.append(str(e) + '\n\n' + traceback.format_exc())
                    raise
            return ret
        return wrapped
    return wrapper



import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

setup(
    name='django-reportmail',
    version='1.0',
    packages=['reportmail'],
    url='https://github.com/hirokiky/django-reportmail',
    license='MIT',
    author='hirokiky',
    author_email='hirokiky@gmail.com',
    description='django library to render and send report mail. ',
    long_description=README + '\n\n' + CHANGES,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Environment :: Web Environment",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Framework :: Django",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        ],
    install_requires=[
        'Django>=1.6,<1.7',
    ],
    include_package_data=True,
    zip_safe=False,
)
